@@ -14,7 +14,7 @@
 #include <fcntl.h>
 
 #include <config_parser.h>
-#include <ksmbdtools.h>
+#include <tools.h>
 
 #include <management/share.h>
 
@@ -29,37 +29,27 @@ static size_t wsz;
 
 static char *new_group_name(char *name)
 {
-	char *gn;
-
 	if (strchr(name, '['))
 		return name;
 
-	gn = g_malloc(strlen(name) + 3);
-	if (gn)
-		sprintf(gn, "[%s]", name);
-	return gn;
+	return g_strdup_printf("[%s]", name);
 }
 
 static char *aux_group_name(char *name)
 {
-	char *gn;
-
-	gn = g_malloc(strlen(name) + 3 + strlen(AUX_GROUP_PREFIX));
-	if (gn)
-		sprintf(gn, "[%s%s]", AUX_GROUP_PREFIX, name);
-	return gn;
+	return g_strdup_printf("[%s%s]", AUX_GROUP_PREFIX, name);
 }
 
 static int __open_smbconf(char *smbconf)
 {
 	conf_fd = open(smbconf, O_WRONLY);
 	if (conf_fd == -1) {
-		pr_err("%s %s\n", strerr(errno), smbconf);
+		pr_err("Can't open `%s': %m\n", smbconf);
 		return -EINVAL;
 	}
 
-	if (ftruncate(conf_fd, 0)) {
-		pr_err("%s %s\n", strerr(errno), smbconf);
+	if (ftruncate(conf_fd, 0) == -1) {
+		pr_err("Can't truncate `%s': %m\n", smbconf);
 		close(conf_fd);
 		return -EINVAL;
 	}
@@ -76,7 +66,7 @@ static void __write(void)
 		if (ret == -1) {
 			if (errno == EINTR)
 				continue;
-			pr_err("%s\n", strerr(errno));
+			pr_err("Failed to write share entry: %m\n");
 			exit(EXIT_FAILURE);
 		}
 
@@ -92,9 +82,8 @@ static void __write_share(gpointer key, gpointer value, gpointer buf)
 
 	wsz = snprintf(wbuf, sizeof(wbuf), "\t%s = %s\n", k, v);
 	if (wsz > sizeof(wbuf)) {
-		pr_err("smb.conf entry size is above the limit: %zu > %zu\n",
-			wsz,
-			sizeof(wbuf));
+		pr_err("Share entry size is above limit: %zu > %zu\n",
+		       wsz, sizeof(wbuf));
 		exit(EXIT_FAILURE);
 	}
 	__write();
@@ -124,8 +113,8 @@ static void write_remove_share_cb(gpointer key,
 {
 	struct smbconf_group *g = (struct smbconf_group *)value;
 
-	if (!g_ascii_strcasecmp(g->name, name)) {
-		pr_info("share '%s' removed\n", g->name);
+	if (shm_share_name_equal(g->name, name)) {
+		pr_info("Share `%s' removed\n", (char *)name);
 		return;
 	}
 
@@ -149,72 +138,71 @@ static void update_share_cb(gpointer key,
 
 int command_add_share(char *smbconf, char *name, char *opts)
 {
-	char *new_name = NULL;
+	g_autofree char *new_name = NULL;
 
 	if (g_hash_table_lookup(parser.groups, name)) {
-		pr_err("Share already exists: %s\n", name);
+		pr_err("Share `%s' already exists\n", name);
 		return -EEXIST;
 	}
 
 	new_name = new_group_name(name);
 	if (cp_parse_external_smbconf_group(new_name, opts))
-		goto error;
+		return -EINVAL;
 
 	if (__open_smbconf(smbconf))
-		goto error;
+		return -EINVAL;
+
+	pr_info("Adding share `%s'\n", name);
 	g_hash_table_foreach(parser.groups, write_share_cb, NULL);
 	close(conf_fd);
-	g_free(new_name);
 	return 0;
 
-error:
-	g_free(new_name);
-	return -EINVAL;
 }
 
 int command_update_share(char *smbconf, char *name, char *opts)
 {
 	struct smbconf_group *existing_group;
 	struct smbconf_group *update_group;
-	char *aux_name = NULL;
+	g_autofree char *aux_name = NULL;
 
 	existing_group = g_hash_table_lookup(parser.groups, name);
 	if (!existing_group) {
-		pr_err("Unknown share: %s\n", name);
-		goto error;
+		pr_err("Share `%s' does not exist\n", name);
+		return -EINVAL;
 	}
 
 	aux_name = aux_group_name(name);
 	if (cp_parse_external_smbconf_group(aux_name, opts))
-		goto error;
+		return -EINVAL;
 
 	/* get rid of [] */
 	sprintf(aux_name, "%s%s", AUX_GROUP_PREFIX, name);
 	update_group = g_hash_table_lookup(parser.groups, aux_name);
 	if (!update_group) {
-		pr_err("Cannot find the external group\n");
-		goto error;
+		pr_err("External group `%s' does not exist\n", aux_name);
+		return -EINVAL;
 	}
 
+	g_free(existing_group->name);
+	existing_group->name = g_strdup(name);
+
 	g_hash_table_foreach(update_group->kv,
 			     update_share_cb,
 			     existing_group->kv);
 
 	if (__open_smbconf(smbconf))
-		goto error;
+		return -EINVAL;
 
+	pr_info("Updating share `%s'\n", name);
 	g_hash_table_foreach(parser.groups, write_share_cb, NULL);
 	close(conf_fd);
-	g_free(aux_name);
 	return 0;
-
-error:
-	g_free(aux_name);
-	return -EINVAL;
 }
 
-int command_del_share(char *smbconf, char *name)
+int command_del_share(char *smbconf, char *name, char *unused)
 {
+	(void)unused;
+
 	if (__open_smbconf(smbconf))
 		return -EINVAL;
 
