@@ -8,8 +8,12 @@
 #ifndef __KSMBD_USER_ADMIN_H__
 #define __KSMBD_USER_ADMIN_H__
 
+#define MAX_NT_PWD_LEN 129
+
+typedef int (*command_fn)(char *, char *, char *);
+
 int command_add_user(char *pwddb, char *account, char *password);
 int command_update_user(char *pwddb, char *account, char *password);
-int command_del_user(char *pwddb, char *account);
+int command_del_user(char *pwddb, char *account, char *unused);
 
 #endif /* __KSMBD_USER_ADMIN_H__ */
