@@ -10,7 +10,12 @@
 
 #include <glib.h>
 
+#define GROUPS_CALLBACK_NONE	(0)
+#define GROUPS_CALLBACK_INIT	(1 << 0)
+#define GROUPS_CALLBACK_REINIT	(1 << 1)
+
 struct smbconf_group {
+	unsigned short		cb_mode;
 	char			*name;
 	GHashTable		*kv;
 };
@@ -29,7 +34,7 @@ void cp_smbconfig_destroy(void);
 int cp_parse_pwddb(const char *pwddb);
 int cp_parse_smbconf(const char *smbconf);
 int cp_parse_reload_smbconf(const char *smbconf);
-int cp_parse_subauth(const char *subauth_path);
+int cp_parse_subauth(void);
 
 char *cp_ltrim(char *v);
 int cp_key_cmp(char *k, char *v);
