@@ -24,9 +24,9 @@ void spnego_destroy(void);
 int spnego_handle_authen_request(struct ksmbd_spnego_authen_request *req,
 				struct ksmbd_spnego_auth_out *auth_out);
 #else
-static int spnego_init(void) { return 0; }
-static void spnego_destroy(void) {}
-static int spnego_handle_authen_request(struct ksmbd_spnego_authen_request *req,
+static inline int spnego_init(void) { return 0; }
+static inline void spnego_destroy(void) {}
+static inline int spnego_handle_authen_request(struct ksmbd_spnego_authen_request *req,
 				struct ksmbd_spnego_auth_out *auth_out)
 {
 	return -ENOTSUP;
