@@ -5,7 +5,7 @@
  *   linux-cifsd-devel@lists.sourceforge.net
  */
 
-#include <ksmbdtools.h>
+#include <tools.h>
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
@@ -32,111 +32,100 @@
 #include "management/spnego.h"
 #include "version.h"
 
-static int no_detach = 0;
-int ksmbd_health_status;
+static int no_detach;
 static pid_t worker_pid;
 static int lock_fd = -1;
 
 typedef int (*worker_fn)(void);
 
-static void usage(void)
+static void usage(int status)
 {
-	fprintf(stderr, "Usage: ksmbd\n");
-	fprintf(stderr, "\t--p=NUM | --port=NUM              TCP port NUM\n");
-	fprintf(stderr, "\t--c=smb.conf | --config=smb.conf  config file\n");
-	fprintf(stderr, "\t--u=pwd.db | --users=pwd.db       Users DB\n");
-	fprintf(stderr, "\t--n | --nodetach                  Don't detach\n");
-	fprintf(stderr, "\t--s | --systemd                   Service mode\n");
-	fprintf(stderr, "\t-V | --version                    Show version\n");
-	fprintf(stderr, "\t-h | --help                       Show help\n");
+	g_printerr(
+		"Usage: ksmbd.mountd [-v] [-p PORT] [-c SMBCONF] [-u PWDDB] [-n[WAY]]\n"
+		"       ksmbd.mountd {-V | -h}\n");
 
-	exit(EXIT_FAILURE);
+	if (status != EXIT_SUCCESS)
+		g_printerr("Try `ksmbd.mountd --help' for more information.\n");
+	else
+		g_printerr(
+			"\n"
+			"  -p, --port=PORT         bind to PORT instead of TCP port " STR(KSMBD_CONF_DEFAULT_TCP_PORT) "\n"
+			"  -c, --config=SMBCONF    use SMBCONF as configuration file instead of\n"
+			"                          `" PATH_SMBCONF "'\n"
+			"  -u, --users=PWDDB       use PWDDB as user database instead of\n"
+			"                          `" PATH_PWDDB "'\n"
+			"  -n, --nodetach[=WAY]    do not detach process from foreground;\n"
+			"                          if WAY is 1, become process group leader (default);\n"
+			"                          if WAY is 0, detach\n"
+			"  -v, --verbose           be verbose\n"
+			"  -V, --version           output version information and exit\n"
+			"  -h, --help              display this help and exit\n"
+			"\n"
+			"See ksmbd.mountd(8) for more details.\n");
 }
 
-static void show_version(void)
+static struct option opts[] = {
+	{"port",	required_argument,	NULL,	'p' },
+	{"config",	required_argument,	NULL,	'c' },
+	{"users",	required_argument,	NULL,	'u' },
+	{"nodetach",	optional_argument,	NULL,	'n' },
+	{"verbose",	no_argument,		NULL,	'v' },
+	{"version",	no_argument,		NULL,	'V' },
+	{"help",	no_argument,		NULL,	'h' },
+	{NULL,		0,			NULL,	 0  }
+};
+
+static int show_version(void)
 {
-	printf("ksmbd-tools version : %s\n", KSMBD_TOOLS_VERSION);
-	exit(EXIT_FAILURE);
-}
-
-static int handle_orphaned_lock_file(void)
-{
-	char proc_ent[64] = {0, };
-	char manager_pid[10] = {0, };
-	int pid = 0;
-	int fd;
-
-	fd = open(KSMBD_LOCK_FILE, O_RDONLY);
-	if (fd < 0)
-		return -EINVAL;
-
-	if (read(fd, &manager_pid, sizeof(manager_pid)) == -1) {
-		pr_debug("Unable to read main PID: %s\n", strerr(errno));
-		close(fd);
-		return -EINVAL;
-	}
-
-	close(fd);
-
-	pid = strtol(manager_pid, NULL, 10);
-	snprintf(proc_ent, sizeof(proc_ent), "/proc/%d", pid);
-	fd = open(proc_ent, O_RDONLY);
-	if (fd < 0) {
-		pr_info("Unlink orphaned '%s'\n", KSMBD_LOCK_FILE);
-		return unlink(KSMBD_LOCK_FILE);
-	}
-
-	close(fd);
-	pr_info("File '%s' belongs to pid %d\n", KSMBD_LOCK_FILE, pid);
-	return -EINVAL;
+	g_print("ksmbd-tools version : %s\n", KSMBD_TOOLS_VERSION);
+	return 0;
 }
 
 static int create_lock_file(void)
 {
-	char manager_pid[10];
+	int ret = -EINVAL;
+	char *open_m = NULL;
+	char pid_buf[10];
 	size_t sz;
 
 retry:
 	lock_fd = open(KSMBD_LOCK_FILE, O_CREAT | O_EXCL | O_WRONLY,
 			S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
 	if (lock_fd < 0) {
-		if (handle_orphaned_lock_file())
-			return -EINVAL;
+		open_m = g_strdup_printf("%m");
+
+		if (send_signal_to_ksmbd_mountd(0) == -ESRCH) {
+			pr_info("Unlinking orphaned lock file\n");
+			if (unlink(KSMBD_LOCK_FILE) == -1) {
+				pr_err("Can't unlink `%s': %m\n", KSMBD_LOCK_FILE);
+				goto out;
+			}
+		} else {
+			pr_debug("Can't create `%s': %s\n", KSMBD_LOCK_FILE,
+				 open_m ? open_m : "Out of memory");
+			goto out;
+		}
+
+		g_free(open_m);
+		open_m = NULL;
 		goto retry;
 	}
 
-	if (flock(lock_fd, LOCK_EX | LOCK_NB) != 0)
-		return -EINVAL;
-
-	sz = snprintf(manager_pid, sizeof(manager_pid), "%d", getpid());
-	if (write(lock_fd, manager_pid, sz) == -1)
-		pr_err("Unable to record main PID: %s\n", strerr(errno));
-	return 0;
-}
-
-char *make_path_subauth(void)
-{
-	char *path;
-	int smbconf_len = strlen(global_conf.smbconf);
-	int loc = 0;
-	const char *subauth_filename = "ksmbd.subauth";
-
-	if (strchr(global_conf.smbconf, '/')) {
-		for (loc = smbconf_len - 1; loc > 0; loc--)
-			if (global_conf.smbconf[loc] == '/') {
-				loc++;
-				break;
-			}
+	if (flock(lock_fd, LOCK_EX | LOCK_NB) == -1) {
+		pr_err("Can't apply exclusive lock: %m\n");
+		goto out;
 	}
 
-	path = calloc(1, loc + strlen(subauth_filename) + 1);
-	if (!path)
-		return NULL;
+	sz = snprintf(pid_buf, sizeof(pid_buf), "%d", getpid());
+	if (write(lock_fd, pid_buf, sz) == -1) {
+		pr_err("Can't write manager PID: %m\n");
+		goto out;
+	}
 
-	strncat(path, global_conf.smbconf, loc);
-	strcat(path, subauth_filename);
-
-	return path;
+	ret = 0;
+out:
+	g_free(open_m);
+	return ret;
 }
 
 /*
@@ -146,76 +135,62 @@ char *make_path_subauth(void)
  */
 static int write_file_safe(char *path, char *buff, size_t length, int mode)
 {
-	int fd, ret = -1;
-	char *path_tmp = g_strdup_printf("%s.tmp", path);
+	int fd;
+	g_autofree char *path_tmp = g_strdup_printf("%s.tmp", path);
 
 	if (g_file_test(path_tmp, G_FILE_TEST_EXISTS))
 		unlink(path_tmp);
 
 	fd = open(path_tmp, O_CREAT | O_EXCL | O_WRONLY, mode);
 	if (fd < 0) {
-		pr_err("Unable to create %s: %s\n", path_tmp, strerr(errno));
-		goto err_out;
+		pr_err("Can't create `%s': %m\n", path_tmp);
+		return -EINVAL;
 	}
 
 	if (write(fd, buff, length) == -1) {
-		pr_err("Unable to write to %s: %s\n", path_tmp, strerr(errno));
+		pr_err("Can't write `%s': %m\n", path_tmp);
 		close(fd);
-		goto err_out;
+		return -EINVAL;
 	}
 
 	fsync(fd);
 	close(fd);
 
-	if (rename(path_tmp, path)) {
-		pr_err("Unable to rename to %s: %s\n", path, strerr(errno));
-		goto err_out;
+	if (rename(path_tmp, path) == -1) {
+		pr_err("Can't rename `%s' to `%s': %m\n", path_tmp, path);
+		return -EINVAL;
 	}
-	ret = 0;
 
-err_out:
-	g_free(path_tmp);
-	return ret;
+	return 0;
 }
 
-static int create_subauth_file(char *path_subauth)
+static int create_subauth_file(void)
 {
-	char subauth_buf[35];
-	GRand *rnd;
-
-	rnd = g_rand_new();
-	sprintf(subauth_buf, "%d:%d:%d\n", g_rand_int_range(rnd, 0, INT_MAX),
+	GRand *rnd = g_rand_new();
+	g_autofree char *subauth_buf = g_strdup_printf("%d:%d:%d\n", g_rand_int_range(rnd, 0, INT_MAX),
 		g_rand_int_range(rnd, 0, INT_MAX),
 		g_rand_int_range(rnd, 0, INT_MAX));
 
-	return write_file_safe(path_subauth, subauth_buf, strlen(subauth_buf),
+	return write_file_safe(PATH_SUBAUTH, subauth_buf, strlen(subauth_buf),
 		S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
 }
 
 static int generate_sub_auth(void)
 {
-	int rc = -EINVAL;
-	char *path_subauth;
+	int ret = -EINVAL;
 
-	path_subauth = make_path_subauth();
-	if (!path_subauth)
-		return -ENOMEM;
 retry:
-	if (g_file_test(path_subauth, G_FILE_TEST_EXISTS)) {
-		rc = cp_parse_subauth(path_subauth);
-	}
+	if (g_file_test(PATH_SUBAUTH, G_FILE_TEST_EXISTS))
+		ret = cp_parse_subauth();
 
-	if (rc < 0) {
-		rc = create_subauth_file(path_subauth);
-		if (rc) {
-			free(path_subauth);
-			return -1;
-		}
+	if (ret) {
+		ret = create_subauth_file();
+		if (ret)
+			return ret;
 		goto retry;
 	}
 
-	free(path_subauth);
-	return 0;
+	return ret;
 }
 
 static void delete_lock_file(void)
@@ -231,18 +206,20 @@ static void delete_lock_file(void)
 
 static int wait_group_kill(int signo)
 {
-	pid_t pid;
+	pid_t child;
 	int status;
 
-	if (kill(worker_pid, signo) != 0)
-		pr_err("can't execute kill %d: %s\n",
-			worker_pid,
-			strerr(errno));
+	if (kill(worker_pid, signo) == -1)
+		pr_err("Unable to send signal %d (%s) to PID %d: %m\n",
+				signo, strsignal(signo), worker_pid);
 
 	while (1) {
-		pid = waitpid(-1, &status, 0);
-		if (pid != 0) {
-			pr_debug("detected pid %d termination\n", pid);
+		child = waitpid(-1, &status, 0);
+		if (child == -1) {
+			pr_debug("waitpid() returned an error: %m\n");
+			break;
+		} else if (child != 0) {
+			pr_debug("Detected state change of PID %d\n", child);
 			break;
 		}
 		sleep(1);
@@ -263,8 +240,8 @@ static int setup_signal_handler(int signo, sighandler_t handler)
 
 	status = sigaction(signo, &act, NULL);
 	if (status != 0)
-		pr_err("Unable to register %s signal handler: %s",
-				strsignal(signo), strerr(errno));
+		pr_err("Unable to register handler for signal %d (%s): %m",
+				signo, strsignal(signo));
 	return status;
 }
 
@@ -288,25 +265,23 @@ static int setup_signals(sighandler_t handler)
 	return 0;
 }
 
-static int parse_configs(char *pwddb, char *smbconf)
+static int parse_configs(void)
 {
 	int ret;
 
-	ret = cp_parse_pwddb(pwddb);
+	ret = cp_parse_pwddb(global_conf.pwddb);
 	if (ret == -ENOENT) {
-		pr_err("User database file does not exist. %s\n",
-			"Only guest sessions (if permitted) will work.");
+		pr_info("User database does not exist, "
+			"only guest sessions may work\n");
 	} else if (ret) {
-		pr_err("Unable to parse user database\n");
+		pr_err("Failed to parse user database\n");
 		return ret;
 	}
 
-	ret = cp_parse_smbconf(smbconf);
-	if (ret) {
-		pr_err("Unable to parse smb configuration file\n");
-		return ret;
-	}
-	return 0;
+	ret = cp_parse_smbconf(global_conf.smbconf);
+	if (ret)
+		pr_err("Failed to parse configuration file\n");
+	return ret;
 }
 
 static void worker_process_free(void)
@@ -324,7 +299,7 @@ static void worker_process_free(void)
 	usm_destroy();
 }
 
-static void child_sig_handler(int signo)
+static void worker_sig_handler(int signo)
 {
 	static volatile int fatal_delivered = 0;
 
@@ -335,12 +310,11 @@ static void child_sig_handler(int signo)
 		 * the configs.
 		 */
 		ksmbd_health_status |= KSMBD_SHOULD_RELOAD_CONFIG;
-		pr_debug("Scheduled a config reload action.\n");
+		pr_debug("Scheduled a config reload action\n");
 		return;
 	}
 
-	pr_info("Child received signal: %d (%s)\n",
-		signo, strsignal(signo));
+	pr_info("Worker received signal %d (%s)\n", signo, strsignal(signo));
 
 	if (!g_atomic_int_compare_and_exchange(&fatal_delivered, 0, 1))
 		return;
@@ -360,15 +334,15 @@ static void manager_sig_handler(int signo)
 			return;
 
 		ksmbd_health_status |= KSMBD_SHOULD_RELOAD_CONFIG;
-		if (kill(worker_pid, signo))
-			pr_err("Unable to send SIGHUP to %d: %s\n",
-				worker_pid, strerr(errno));
+		if (kill(worker_pid, signo) == -1)
+			pr_err("Unable to send signal %d (%s) to PID %d: %m\n",
+					signo, strsignal(signo), worker_pid);
 		return;
 	}
 
 	setup_signals(SIG_DFL);
 	wait_group_kill(signo);
-	pr_info("Exiting. Bye!\n");
+	pr_info("Exiting, bye!\n");
 	delete_lock_file();
 	kill(0, SIGINT);
 }
@@ -377,7 +351,7 @@ static int worker_process_init(void)
 {
 	int ret;
 
-	setup_signals(child_sig_handler);
+	setup_signals(worker_sig_handler);
 	set_logger_app_name("ksmbd-worker");
 
 	ret = usm_init();
@@ -392,11 +366,9 @@ static int worker_process_init(void)
 		goto out;
 	}
 
-	ret = parse_configs(global_conf.pwddb, global_conf.smbconf);
-	if (ret) {
-		pr_err("Failed to parse configuration files\n");
+	ret = parse_configs();
+	if (ret)
 		goto out;
-	}
 
 	ret = sm_init();
 	if (ret) {
@@ -424,7 +396,7 @@ static int worker_process_init(void)
 
 	ret = spnego_init();
 	if (ret) {
-		pr_err("Failed to init spnego subsystem\n");
+		pr_err("Failed to init SPNEGO subsystem\n");
 		ret = KSMBD_STATUS_IPC_FATAL_ERROR;
 		goto out;
 	}
@@ -448,11 +420,11 @@ static pid_t start_worker_process(worker_fn fn)
 
 	__pid = fork();
 	if (__pid < 0) {
-		pr_err("Can't fork child process: `%s'\n", strerr(errno));
+		pr_err("Can't fork worker process: %m\n");
 		return -EINVAL;
 	}
 	if (__pid == 0) {
-		status = fn();
+		status = fn() ? EXIT_FAILURE : EXIT_SUCCESS;
 		exit(status);
 	}
 	return __pid;
@@ -472,24 +444,16 @@ static int manager_process_init(void)
 			pr_err("Daemonization failed\n");
 			goto out;
 		}
-	} else {
-		/*
-		 * Make ourselves a process group leader; if we are
-		 * the group leader already then the function will do
-		 * nothing (apart from setting errnor to EPERM).
-		 */
-		if (no_detach == 1)
-			setsid();
-	}
+	} else if (no_detach == 1)
+		setpgid(0, 0);
 
 	if (create_lock_file()) {
-		pr_err("Failed to create lock file: %s\n", strerr(errno));
+		pr_err("Failed to create lock file\n");
 		goto out;
 	}
 
 	if (generate_sub_auth())
-		pr_debug("Failed to generate subauth for domain sid: %s\n",
-				strerr(errno));
+		pr_debug("Unable to generate subauth for domain SID: %m\n");
 
 	worker_pid = start_worker_process(worker_process_init);
 	if (worker_pid < 0)
@@ -500,23 +464,26 @@ static int manager_process_init(void)
 		pid_t child;
 
 		child = waitpid(-1, &status, 0);
-		if (ksmbd_health_status & KSMBD_SHOULD_RELOAD_CONFIG &&
-				errno == EINTR) {
-			ksmbd_health_status &= ~KSMBD_SHOULD_RELOAD_CONFIG;
-			continue;
-		}
-
-		pr_err("WARNING: child process exited abnormally: %d\n",
-				child);
-		if (child == -1) {
-			pr_err("waitpid() returned error code: %s\n",
-				strerr(errno));
-			goto out;
-		}
+		if (child == -1)
+			switch (errno) {
+			case EINTR:
+				if (ksmbd_health_status &
+						KSMBD_SHOULD_RELOAD_CONFIG) {
+					ksmbd_health_status &=
+						~KSMBD_SHOULD_RELOAD_CONFIG;
+					continue;
+				}
+				/* Fall through */
+			default:
+				pr_err("waitpid() returned an error: %m\n");
+				goto out;
+			}
+		else if (child != 0)
+			pr_info("Worker PID %d changed state\n", child);
 
 		if (WIFEXITED(status) &&
 			WEXITSTATUS(status) == KSMBD_STATUS_IPC_FATAL_ERROR) {
-			pr_err("Fatal IPC error. Terminating. Check dmesg.\n");
+			pr_err("Fatal IPC error, terminating, check dmesg!\n");
 			goto out;
 		}
 
@@ -532,60 +499,26 @@ out:
 	return 0;
 }
 
-static int manager_systemd_service(void)
+int mountd_main(int argc, char **argv)
 {
-	pid_t __pid;
-
-	__pid = start_worker_process(manager_process_init);
-	if (__pid < 0)
-		return -EINVAL;
-
-	return 0;
-}
-
-static struct option opts[] = {
-	{"port",	required_argument,	NULL,	'p' },
-	{"config",	required_argument,	NULL,	'c' },
-	{"users",	required_argument,	NULL,	'u' },
-	{"systemd",	no_argument,		NULL,	's' },
-	{"nodetach",	optional_argument,	NULL,	'n' },
-	{"help",	no_argument,		NULL,	'h' },
-	{"?",		no_argument,		NULL,	'?' },
-	{"version",	no_argument,		NULL,	'V' },
-	{NULL,		0,			NULL,	 0  }
-};
-
-int main(int argc, char *argv[])
-{
-	int systemd_service = 0;
+	int ret = -EINVAL;
 	int c;
 
 	set_logger_app_name("ksmbd.mountd");
+
 	memset(&global_conf, 0x00, sizeof(struct smbconf_global));
-	global_conf.pwddb = PATH_PWDDB;
-	global_conf.smbconf = PATH_SMBCONF;
-	pr_logger_init(PR_LOGGER_STDIO);
-
-	opterr = 0;
-	while (1) {
-		c = getopt_long(argc, argv, "n::p:c:u:sVh", opts, NULL);
-
-		if (c < 0)
-			break;
-
+	while ((c = getopt_long(argc, argv, "p:c:u:n::vVh", opts, NULL)) != EOF)
 		switch (c) {
-		case 0: /* getopt_long() set a variable, just keep going */
-			break;
-		case 1:
-			break;
 		case 'p':
-			global_conf.tcp_port = cp_get_group_kv_long(optarg);
 			pr_debug("TCP port option override\n");
+			global_conf.tcp_port = cp_get_group_kv_long(optarg);
 			break;
 		case 'c':
+			g_free(global_conf.smbconf);
 			global_conf.smbconf = g_strdup(optarg);
 			break;
 		case 'u':
+			g_free(global_conf.pwddb);
 			global_conf.pwddb = g_strdup(optarg);
 			break;
 		case 'n':
@@ -594,30 +527,51 @@ int main(int argc, char *argv[])
 			else
 				no_detach = cp_get_group_kv_long(optarg);
 			break;
-		case 's':
-			systemd_service = 1;
+		case 'v':
+			set_log_level(PR_DEBUG);
 			break;
 		case 'V':
-			show_version();
-			break;
-		case ':':
-			pr_err("Missing option argument\n");
+			ret = show_version();
+			goto out;
+		case 'h':
+			ret = 0;
 			/* Fall through */
 		case '?':
-		case 'h':
-			/* Fall through */
 		default:
-			usage();
+			usage(ret ? EXIT_FAILURE : EXIT_SUCCESS);
+			goto out;
+		}
+
+	if (argc > optind) {
+		usage(ret ? EXIT_FAILURE : EXIT_SUCCESS);
+		goto out;
+	}
+
+	if (!global_conf.smbconf) {
+		global_conf.smbconf = g_strdup(PATH_SMBCONF);
+		if (!global_conf.smbconf) {
+			pr_err("Out of memory\n");
+			goto out;
+		}
+		if (!g_file_test(global_conf.smbconf, G_FILE_TEST_EXISTS) &&
+		    g_file_test(PATH_SMBCONF_FALLBACK, G_FILE_TEST_EXISTS)) {
+			pr_err("Use of `%s' is deprecated, rename it to `%s' now!\n",
+					PATH_SMBCONF_FALLBACK, PATH_SMBCONF);
+			g_free(global_conf.smbconf);
+			global_conf.smbconf = g_strdup(PATH_SMBCONF_FALLBACK);
 		}
 	}
 
-	if (!global_conf.smbconf || !global_conf.pwddb) {
-		pr_err("Out of memory\n");
-		exit(EXIT_FAILURE);
+	if (!global_conf.pwddb) {
+		global_conf.pwddb = g_strdup(PATH_PWDDB);
+		if (!global_conf.pwddb) {
+			pr_err("Out of memory\n");
+			goto out;
+		}
 	}
 
 	setup_signals(manager_sig_handler);
-	if (!systemd_service)
-		return manager_process_init();
-	return manager_systemd_service();
+	ret = manager_process_init();
+out:
+	return ret ? EXIT_FAILURE : EXIT_SUCCESS;
 }
