@@ -16,7 +16,7 @@
 #include <rpc_wkssvc.h>
 #include <rpc_samr.h>
 #include <rpc_lsarpc.h>
-#include <ksmbdtools.h>
+#include <tools.h>
 
 static GHashTable	*pipes_table;
 static GRWLock		pipes_table_lock;
@@ -111,31 +111,8 @@ static struct ksmbd_rpc_pipe *rpc_pipe_lookup(unsigned int id)
 static void dcerpc_free(struct ksmbd_dcerpc *dce)
 {
 	if (!(dce->flags & KSMBD_DCERPC_EXTERNAL_PAYLOAD))
-		free(dce->payload);
-	free(dce);
-}
-
-static struct ksmbd_dcerpc *dcerpc_alloc(unsigned int flags, int sz)
-{
-	struct ksmbd_dcerpc *dce;
-
-	dce = calloc(1, sizeof(struct ksmbd_dcerpc));
-	if (!dce)
-		return NULL;
-
-	dce->payload = calloc(1, sz);
-	if (!dce->payload) {
-		free(dce);
-		return NULL;
-	}
-
-	dce->payload_sz = sz;
-	dce->flags = flags;
-	dce->num_pointers = 1;
-
-	if (sz == KSMBD_DCERPC_MAX_PREFERRED_SIZE)
-		dce->flags &= ~KSMBD_DCERPC_FIXED_PAYLOAD_SZ;
-	return dce;
+		g_free(dce->payload);
+	g_free(dce);
 }
 
 static struct ksmbd_dcerpc *dcerpc_ext_alloc(unsigned int flags,
@@ -144,7 +121,7 @@ static struct ksmbd_dcerpc *dcerpc_ext_alloc(unsigned int flags,
 {
 	struct ksmbd_dcerpc *dce;
 
-	dce = calloc(1, sizeof(struct ksmbd_dcerpc));
+	dce = g_try_malloc0(sizeof(struct ksmbd_dcerpc));
 	if (!dce)
 		return NULL;
 
@@ -181,8 +158,9 @@ static void __rpc_pipe_free(struct ksmbd_rpc_pipe *pipe)
 	rpc_pipe_reset(pipe);
 	if (pipe->dce)
 		dcerpc_free(pipe->dce);
-	g_array_free(pipe->entries, 1);
-	free(pipe);
+	if (pipe->entries)
+		g_array_free(pipe->entries, 1);
+	g_free(pipe);
 }
 
 static void rpc_pipe_free(struct ksmbd_rpc_pipe *pipe)
@@ -200,7 +178,7 @@ static struct ksmbd_rpc_pipe *rpc_pipe_alloc(void)
 {
 	struct ksmbd_rpc_pipe *pipe;
 
-	pipe = calloc(1, sizeof(struct ksmbd_rpc_pipe));
+	pipe = g_try_malloc0(sizeof(struct ksmbd_rpc_pipe));
 	if (!pipe)
 		return NULL;
 
@@ -347,7 +325,9 @@ int ndr_write_union_##name(struct ksmbd_dcerpc *dce, type value)	\
 	int ret;							\
 									\
 	ret = ndr_write_##name(dce, value);				\
-	ret |= ndr_write_##name(dce, value);				\
+	if (ret)							\
+		return ret;						\
+	ret = ndr_write_##name(dce, value);				\
 	return ret;							\
 }
 
@@ -378,7 +358,7 @@ NDR_READ_UNION(int32, __u32);
 int ndr_write_bytes(struct ksmbd_dcerpc *dce, void *value, size_t sz)
 {
 	align_offset(dce, 2);
-	if (try_realloc_payload(dce, sizeof(short)))
+	if (try_realloc_payload(dce, sz))
 		return -ENOMEM;
 
 	memcpy(PAYLOAD_HEAD(dce), value, sz);
@@ -421,7 +401,7 @@ static gchar *ndr_convert_char_to_unicode(struct ksmbd_dcerpc *dce, char *str,
 
 int ndr_write_vstring(struct ksmbd_dcerpc *dce, void *value)
 {
-	gchar *out;
+	g_autofree char *out = NULL;
 	gsize bytes_written = 0;
 
 	size_t raw_len, str_len;
@@ -450,18 +430,28 @@ int ndr_write_vstring(struct ksmbd_dcerpc *dce, void *value)
 	 * passed, including the terminator.
 	 */
 	ret = ndr_write_int32(dce, str_len);
-	ret |= ndr_write_int32(dce, 0);
-	ret |= ndr_write_int32(dce, str_len);
-	ret |= ndr_write_bytes(dce, out, bytes_written);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, 0);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, str_len);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, out, bytes_written);
+	if (ret)
+		return ret;
 	auto_align_offset(dce);
 
-	g_free(out);
 	return ret;
 }
 
 int ndr_write_string(struct ksmbd_dcerpc *dce, char *str)
 {
-	gchar *out;
+	g_autofree char *out = NULL;
 	gsize bytes_written = 0;
 
 	size_t len;
@@ -476,18 +466,26 @@ int ndr_write_string(struct ksmbd_dcerpc *dce, char *str)
 		return -EINVAL;
 
 	ret = ndr_write_int32(dce, len); // max count
-	ret |= ndr_write_int32(dce, 0);
-	ret |= ndr_write_int32(dce, len); // actual count
-	ret |= ndr_write_bytes(dce, out, bytes_written);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, 0);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, len); // actual count
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, out, bytes_written);
 	auto_align_offset(dce);
 
-	g_free(out);
 	return ret;
 }
 
 int ndr_write_lsa_string(struct ksmbd_dcerpc *dce, char *str)
 {
-	gchar *out;
+	g_autofree char *out = NULL;
 	gsize bytes_written = 0;
 
 	size_t len;
@@ -502,12 +500,20 @@ int ndr_write_lsa_string(struct ksmbd_dcerpc *dce, char *str)
 		return -EINVAL;
 
 	ret = ndr_write_int32(dce, len + 1); // max count
-	ret |= ndr_write_int32(dce, 0);
-	ret |= ndr_write_int32(dce, len); // actual count
-	ret |= ndr_write_bytes(dce, out, bytes_written);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, 0);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, len); // actual count
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, out, bytes_written);
 	auto_align_offset(dce);
 
-	g_free(out);
 	return ret;
 }
 
@@ -674,9 +680,14 @@ static int ndr_write_empty_array_of_struct(struct ksmbd_rpc_pipe *pipe)
 	struct ksmbd_dcerpc *dce = pipe->dce;
 
 	dce->num_pointers++;
-	ndr_write_int32(dce, dce->num_pointers);
-	ndr_write_int32(dce, 0);
-	ndr_write_int32(dce, 0);
+	if (ndr_write_int32(dce, dce->num_pointers))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int32(dce, 0))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int32(dce, 0))
+		return KSMBD_RPC_EBAD_DATA;
 
 	return KSMBD_RPC_OK;
 }
@@ -685,7 +696,6 @@ int ndr_write_array_of_structs(struct ksmbd_rpc_pipe *pipe)
 {
 	struct ksmbd_dcerpc *dce = pipe->dce;
 	int max_entry_nr;
-	int ret = KSMBD_RPC_OK;
 
 	/*
 	 * In the NDR representation of a structure that contains a
@@ -699,10 +709,8 @@ int ndr_write_array_of_structs(struct ksmbd_rpc_pipe *pipe)
 		return ndr_write_empty_array_of_struct(pipe);
 
 	max_entry_nr = __max_entries(dce, pipe);
-	if (max_entry_nr != pipe->num_entries)
-		ret = KSMBD_RPC_EMORE_DATA;
-
-	ndr_write_int32(dce, max_entry_nr);
+	if (ndr_write_int32(dce, max_entry_nr))
+		return KSMBD_RPC_EBAD_DATA;
 	/*
 	 * ARRAY representation [per dimension]
 	 *    max_count
@@ -711,9 +719,14 @@ int ndr_write_array_of_structs(struct ksmbd_rpc_pipe *pipe)
 	 *    element representation [1..N]
 	 *    actual elements [1..N]
 	 */
-	ndr_write_int32(dce, max_entry_nr);
-	ndr_write_int32(dce, 1);
-	ndr_write_int32(dce, max_entry_nr);
+	if (ndr_write_int32(dce, max_entry_nr))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int32(dce, 1))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int32(dce, max_entry_nr))
+		return KSMBD_RPC_EBAD_DATA;
 
 	if (max_entry_nr == 0) {
 		pr_err("DCERPC: can't fit any data, buffer is too small\n");
@@ -730,8 +743,10 @@ int rpc_init(void)
 	if (!pipes_table)
 		return -ENOMEM;
 	g_rw_lock_init(&pipes_table_lock);
-	rpc_samr_init();
-	rpc_lsarpc_init();
+	if (rpc_samr_init())
+		return -ENOMEM;
+	if (rpc_lsarpc_init())
+		return -ENOMEM;
 	return 0;
 }
 
@@ -740,6 +755,7 @@ void rpc_destroy(void)
 	if (pipes_table) {
 		__clear_pipes_table();
 		g_hash_table_destroy(pipes_table);
+		pipes_table = NULL;
 	}
 	g_rw_lock_clear(&pipes_table_lock);
 	rpc_samr_destroy();
@@ -749,15 +765,40 @@ void rpc_destroy(void)
 static int dcerpc_hdr_write(struct ksmbd_dcerpc *dce,
 			    struct dcerpc_header *hdr)
 {
-	ndr_write_int8(dce, hdr->rpc_vers);
-	ndr_write_int8(dce, hdr->rpc_vers_minor);
-	ndr_write_int8(dce, hdr->ptype);
-	ndr_write_int8(dce, hdr->pfc_flags);
-	ndr_write_bytes(dce, &hdr->packed_drep, sizeof(hdr->packed_drep));
-	ndr_write_int16(dce, hdr->frag_length);
-	ndr_write_int16(dce, hdr->auth_length);
-	ndr_write_int32(dce, hdr->call_id);
-	return 0;
+	int ret;
+
+	ret = ndr_write_int8(dce, hdr->rpc_vers);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int8(dce, hdr->rpc_vers_minor);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int8(dce, hdr->ptype);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int8(dce, hdr->pfc_flags);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, &hdr->packed_drep,
+			      sizeof(hdr->packed_drep));
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, hdr->frag_length);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, hdr->auth_length);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int32(dce, hdr->call_id);
+
+	return ret;
 }
 
 static int dcerpc_hdr_read(struct ksmbd_dcerpc *dce,
@@ -804,11 +845,20 @@ static int dcerpc_hdr_read(struct ksmbd_dcerpc *dce,
 static int dcerpc_response_hdr_write(struct ksmbd_dcerpc *dce,
 				     struct dcerpc_response_header *hdr)
 {
-	ndr_write_int32(dce, hdr->alloc_hint);
-	ndr_write_int16(dce, hdr->context_id);
-	ndr_write_int8(dce, hdr->cancel_count);
+	int ret;
+
+	ret = ndr_write_int32(dce, hdr->alloc_hint);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, hdr->context_id);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int8(dce, hdr->cancel_count);
 	auto_align_offset(dce);
-	return 0;
+
+	return ret;
 }
 
 static int dcerpc_request_hdr_read(struct ksmbd_dcerpc *dce,
@@ -825,7 +875,7 @@ static int dcerpc_request_hdr_read(struct ksmbd_dcerpc *dce,
 
 int dcerpc_write_headers(struct ksmbd_dcerpc *dce, int method_status)
 {
-	int payload_offset;
+	int payload_offset, ret;
 
 	payload_offset = dce->offset;
 	dce->offset = 0;
@@ -835,13 +885,17 @@ int dcerpc_write_headers(struct ksmbd_dcerpc *dce, int method_status)
 	dce->hdr.frag_length = payload_offset;
 	if (method_status == KSMBD_RPC_EMORE_DATA)
 		dce->hdr.pfc_flags = 0;
-	dcerpc_hdr_write(dce, &dce->hdr);
+	ret = dcerpc_hdr_write(dce, &dce->hdr);
+	if (ret)
+		return ret;
 
 	/* cast req_hdr to resp_hdr and NULL out lower 2 bytes */
 	dce->req_hdr.opnum = 0;
 	dce->resp_hdr.cancel_count = 0;
 	dce->resp_hdr.alloc_hint = payload_offset;
-	dcerpc_response_hdr_write(dce, &dce->resp_hdr);
+	ret = dcerpc_response_hdr_write(dce, &dce->resp_hdr);
+	if (ret)
+		return ret;
 
 	dce->offset = payload_offset;
 	return 0;
@@ -871,14 +925,36 @@ static int __dcerpc_read_syntax(struct ksmbd_dcerpc *dce,
 static int __dcerpc_write_syntax(struct ksmbd_dcerpc *dce,
 				 struct dcerpc_syntax *syn)
 {
-	ndr_write_int32(dce, syn->uuid.time_low);
-	ndr_write_int16(dce, syn->uuid.time_mid);
-	ndr_write_int16(dce, syn->uuid.time_hi_and_version);
-	ndr_write_bytes(dce, syn->uuid.clock_seq, sizeof(syn->uuid.clock_seq));
-	ndr_write_bytes(dce, syn->uuid.node, sizeof(syn->uuid.node));
-	ndr_write_int16(dce, syn->ver_major);
-	ndr_write_int16(dce, syn->ver_minor);
-	return 0;
+	int ret;
+
+	ret = ndr_write_int32(dce, syn->uuid.time_low);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, syn->uuid.time_mid);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, syn->uuid.time_hi_and_version);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, syn->uuid.clock_seq,
+			      sizeof(syn->uuid.clock_seq));
+	if (ret)
+		return ret;
+
+	ret = ndr_write_bytes(dce, syn->uuid.node, sizeof(syn->uuid.node));
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, syn->ver_major);
+	if (ret)
+		return ret;
+
+	ret = ndr_write_int16(dce, syn->ver_minor);
+
+	return ret;
 }
 
 static void dcerpc_bind_req_free(struct dcerpc_bind_request *hdr)
@@ -886,8 +962,8 @@ static void dcerpc_bind_req_free(struct dcerpc_bind_request *hdr)
 	int i;
 
 	for (i = 0; i < hdr->num_contexts; i++)
-		free(hdr->list[i].transfer_syntaxes);
-	free(hdr->list);
+		g_free(hdr->list[i].transfer_syntaxes);
+	g_free(hdr->list);
 	hdr->list = NULL;
 	hdr->num_contexts = 0;
 }
@@ -913,7 +989,7 @@ static int dcerpc_parse_bind_req(struct ksmbd_dcerpc *dce,
 	if (!hdr->num_contexts)
 		return 0;
 
-	hdr->list = calloc(hdr->num_contexts, sizeof(struct dcerpc_context));
+	hdr->list = g_try_malloc0_n(hdr->num_contexts, sizeof(struct dcerpc_context));
 	if (!hdr->list)
 		return -ENOMEM;
 
@@ -931,7 +1007,7 @@ static int dcerpc_parse_bind_req(struct ksmbd_dcerpc *dce,
 
 		__dcerpc_read_syntax(dce, &ctx->abstract_syntax);
 
-		ctx->transfer_syntaxes = calloc(ctx->num_syntaxes,
+		ctx->transfer_syntaxes = g_try_malloc0_n(ctx->num_syntaxes,
 						sizeof(struct dcerpc_syntax));
 		if (!ctx->transfer_syntaxes) {
 			ret = -ENOMEM;
@@ -944,7 +1020,7 @@ static int dcerpc_parse_bind_req(struct ksmbd_dcerpc *dce,
 	return KSMBD_RPC_OK;
 
 fail:
-	free(hdr->list);
+	g_free(hdr->list);
 	return ret;
 }
 
@@ -993,14 +1069,21 @@ static int dcerpc_bind_nack_return(struct ksmbd_rpc_pipe *pipe)
 
 	dce->offset = sizeof(struct dcerpc_header);
 
-	ndr_write_int16(dce,
-			DCERPC_BIND_NAK_RSN_PROTOCOL_VERSION_NOT_SUPPORTED);
-	ndr_write_int8(dce, ARRAY_SIZE(known_syntaxes));
+	if (ndr_write_int16(dce,
+			    DCERPC_BIND_NAK_RSN_PROTOCOL_VERSION_NOT_SUPPORTED))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int8(dce, ARRAY_SIZE(known_syntaxes)))
+		return KSMBD_RPC_EBAD_DATA;
+
 	auto_align_offset(dce);
 
 	for (i = 0; i < ARRAY_SIZE(known_syntaxes); i++) {
-		ndr_write_int8(dce, known_syntaxes[i].syn.ver_major);
-		ndr_write_int8(dce, known_syntaxes[i].syn.ver_minor);
+		if (ndr_write_int8(dce, known_syntaxes[i].syn.ver_major))
+			return KSMBD_RPC_EBAD_DATA;
+
+		if (ndr_write_int8(dce, known_syntaxes[i].syn.ver_minor))
+			return KSMBD_RPC_EBAD_DATA;
 	}
 
 	payload_offset = dce->offset;
@@ -1009,7 +1092,8 @@ static int dcerpc_bind_nack_return(struct ksmbd_rpc_pipe *pipe)
 	dce->hdr.ptype = DCERPC_PTYPE_RPC_BINDNACK;
 	dce->hdr.pfc_flags = DCERPC_PFC_FIRST_FRAG | DCERPC_PFC_LAST_FRAG;
 	dce->hdr.frag_length = payload_offset;
-	dcerpc_hdr_write(dce, &dce->hdr);
+	if (dcerpc_hdr_write(dce, &dce->hdr))
+		return KSMBD_RPC_EBAD_DATA;
 
 	dce->offset = payload_offset;
 	dce->rpc_resp->payload_sz = dce->offset;
@@ -1030,9 +1114,14 @@ static int dcerpc_bind_ack_return(struct ksmbd_rpc_pipe *pipe)
 	if (dce->bi_req.assoc_group_id == 0)
 		dce->bi_req.assoc_group_id = 0x53f0;
 
-	ndr_write_int16(dce, dce->bi_req.max_xmit_frag_sz);
-	ndr_write_int16(dce, dce->bi_req.max_recv_frag_sz);
-	ndr_write_int32(dce, dce->bi_req.assoc_group_id);
+	if (ndr_write_int16(dce, dce->bi_req.max_xmit_frag_sz))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int16(dce, dce->bi_req.max_recv_frag_sz))
+		return KSMBD_RPC_EBAD_DATA;
+
+	if (ndr_write_int32(dce, dce->bi_req.assoc_group_id))
+		return KSMBD_RPC_EBAD_DATA;
 
 	if (dce->bi_req.flags & KSMBD_RPC_SRVSVC_METHOD_INVOKE)
 		addr = "\\PIPE\\srvsvc";
@@ -1046,16 +1135,24 @@ static int dcerpc_bind_ack_return(struct ksmbd_rpc_pipe *pipe)
 		return KSMBD_RPC_EBAD_FUNC;
 
 	if (dce->hdr.ptype == DCERPC_PTYPE_RPC_ALTCONT) {
-		ndr_write_int16(dce, 0);
-		ndr_write_int16(dce, 0);
+		if (ndr_write_int16(dce, 0))
+			return KSMBD_RPC_EBAD_DATA;
+
+		if (ndr_write_int16(dce, 0))
+			return KSMBD_RPC_EBAD_DATA;
 	} else {
-		ndr_write_int16(dce, strlen(addr));
-		ndr_write_bytes(dce, addr, strlen(addr));
+		if (ndr_write_int16(dce, strlen(addr)))
+			return KSMBD_RPC_EBAD_DATA;
+
+		if (ndr_write_bytes(dce, addr, strlen(addr)))
+			return KSMBD_RPC_EBAD_DATA;
 	}
 	align_offset(dce, 4); /* [flag(NDR_ALIGN4)]    DATA_BLOB _pad1; */
 
 	num_trans = dce->bi_req.num_contexts;
-	ndr_write_int8(dce, num_trans);
+	if (ndr_write_int8(dce, num_trans))
+		return KSMBD_RPC_EBAD_DATA;
+
 	align_offset(dce, 2);
 
 	for (i = 0; i < num_trans; i++) {
@@ -1067,17 +1164,24 @@ static int dcerpc_bind_ack_return(struct ksmbd_rpc_pipe *pipe)
 
 		if (result == -1) {
 			result = DCERPC_BIND_ACK_RES_PROVIDER_REJECT;
-			ndr_write_union_int16(dce, result);
+			if (ndr_write_union_int16(dce, result))
+				return KSMBD_RPC_EBAD_DATA;
 		} else {
-			if (result == DCERPC_BIND_ACK_RES_ACCEPT)
-				ndr_write_union_int16(dce, result);
+			if (result == DCERPC_BIND_ACK_RES_ACCEPT) {
+				if (ndr_write_union_int16(dce, result))
+					return KSMBD_RPC_EBAD_DATA;
+			}
 			if (result == DCERPC_BIND_ACK_RES_NEGOTIATE_ACK) {
-				ndr_write_int16(dce, result);
-				ndr_write_int16(dce, 0x00);
+				if (ndr_write_int16(dce, result))
+					return KSMBD_RPC_EBAD_DATA;
+
+				if (ndr_write_int16(dce, 0x00))
+					return KSMBD_RPC_EBAD_DATA;
 				s = &negotiate_ack_PNIO_uuid;
 			}
 		}
-		__dcerpc_write_syntax(dce, s);
+		if (__dcerpc_write_syntax(dce, s))
+			return KSMBD_RPC_EBAD_DATA;
 	}
 
 	payload_offset = dce->offset;
@@ -1089,7 +1193,8 @@ static int dcerpc_bind_ack_return(struct ksmbd_rpc_pipe *pipe)
 		dce->hdr.ptype = DCERPC_PTYPE_RPC_BINDACK;
 	dce->hdr.pfc_flags = DCERPC_PFC_FIRST_FRAG | DCERPC_PFC_LAST_FRAG;
 	dce->hdr.frag_length = payload_offset;
-	dcerpc_hdr_write(dce, &dce->hdr);
+	if (dcerpc_hdr_write(dce, &dce->hdr))
+		return KSMBD_RPC_EBAD_DATA;
 
 	dce->offset = payload_offset;
 	dce->rpc_resp->payload_sz = dce->offset;
