@@ -9,7 +9,7 @@
 #include <errno.h>
 #include <linux/ksmbd_server.h>
 
-#include <ksmbdtools.h>
+#include <tools.h>
 #include <worker.h>
 #include <ipc.h>
 #include <rpc.h>
@@ -106,7 +106,7 @@ static int spnego_authen_request(struct ksmbd_ipc_msg *msg)
 			sizeof(login_req.account));
 	usm_handle_login_request(&login_req, &resp->login_response);
 	if (!(resp->login_response.status & KSMBD_USER_FLAG_OK)) {
-		pr_info("failed to login %s\n", login_req.account);
+		pr_info("Unable to login user `%s'\n", login_req.account);
 		goto out_free_auth;
 	}
 
@@ -116,7 +116,7 @@ static int spnego_authen_request(struct ksmbd_ipc_msg *msg)
 	memcpy(resp->payload + auth_out.key_len, auth_out.spnego_blob,
 			auth_out.blob_len);
 out_free_auth:
-	free(auth_out.spnego_blob);
+	g_free(auth_out.spnego_blob);
 	free(auth_out.sess_key);
 	free(auth_out.user_name);
 out:
@@ -220,9 +220,12 @@ static int rpc_request(struct ksmbd_ipc_msg *msg)
 {
 	struct ksmbd_rpc_command *req;
 	struct ksmbd_rpc_command *resp;
-	struct ksmbd_ipc_msg *resp_msg;
+	struct ksmbd_ipc_msg *resp_msg = NULL;
 	int ret = -ENOTSUP;
 
+	if (msg->sz < sizeof(struct ksmbd_rpc_command))
+		goto out;
+
 	req = KSMBD_IPC_MSG_PAYLOAD(msg);
 	if (req->flags & KSMBD_RPC_METHOD_RETURN)
 		resp_msg = ipc_msg_alloc(KSMBD_IPC_MAX_MESSAGE_SIZE -
